package com.engine.jucailinkq.attendance.workflow.action.common;

import com.engine.common.util.ServiceUtil;
import com.engine.jucailinkq.attendance.attendanceanalysis.wrapper.AttendaceActionWrapper;
import com.engine.jucailinkq.attendance.workflow.enums.WorkFlowTypeEnum;
import com.engine.jucailinkq.common.util.CommonUtil;
import com.engine.jucailinkq.common.util.DateUtil;
import com.engine.jucailinkq.common.util.DbTools;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import weaver.interfaces.workflow.action.Action;
import weaver.soa.workflow.request.RequestInfo;

import java.time.ZoneOffset;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 流程归档后分析出勤结果
 */
@Slf4j
public class ExcuteAttendanceAnysisAction implements Action {

    private String userKey;

    private AttendaceActionWrapper action = ServiceUtil.getService(AttendaceActionWrapper.class);


    @Override
    public String execute(RequestInfo requestInfo) {
        log.info("userKey : [{}]",userKey);
        String workflowId = requestInfo.getWorkflowid();
        String sql = "select lclx from uf_jcl_kq_kqlcsz where ljid = ?";
        List<Map<String,Object>> workflowSet = DbTools.getSqlToList(sql,workflowId);
        List<Map<String, String>> detailTableData = null;
        if (workflowSet.size() >0){
            WorkFlowTypeEnum worflowType = WorkFlowTypeEnum.getEnum(workflowSet.get(0).get("lclx").toString());
            detailTableData = CommonUtil.getDetailTableInfo(requestInfo,0, worflowType,requestInfo.getWorkflowid());
            if (worflowType == WorkFlowTypeEnum.ASKFORLEAVE || worflowType == WorkFlowTypeEnum.BATCH_ASKFORLEAVE){
                userKey = "qjr";
            }else if (worflowType == WorkFlowTypeEnum.OVERTIME_PLAN){
                userKey = "jbry";
            }else if (worflowType == WorkFlowTypeEnum.EVECTION){
                userKey = "ccr";
            }
        }else {
            detailTableData = CommonUtil.getDetailTableInfo(requestInfo,0);
        }
        Map<String,List<Map<String,String>>> detailGroupUser = detailTableData.stream().collect(Collectors.groupingBy(e->e.get(userKey)));

        for (Map.Entry<String,List<Map<String,String>>> entry :detailGroupUser.entrySet()){
            String userId = entry.getKey();
            List<Map<String,String>> dataList = entry.getValue().stream().sorted(Comparator.comparing(e-> DateUtil.getTime(e.get("ksrq")).toInstant(ZoneOffset.of("+8")).toEpochMilli())).collect(Collectors.toList());
            if (dataList.size() > 0){
                String startDate = dataList.get(0).get("ksrq");
                String endDate = dataList.get(dataList.size()-1).get("jsrq");
                Map<String,Object> paramMap = Maps.newHashMap();
                paramMap.put("startDate",startDate);
                paramMap.put("endDate",endDate);
                paramMap.put("userIds",userId);
                action.alysisActionWrapper(paramMap);
            }
        }

        return null;
    }
}
